# -*- coding: utf-8 -*-
#
# @project TirjaPy
# @file src/tirjapy/base/StoreBase.py
# @author  Shreos Roychowdhury <shreos@tirja.com>
# @version 1.0.0
# 
# @section DESCRIPTION
# 
#   StoreBase.py : Storage Baisc Data Model
# 
# @section LICENSE
# 
# Copyright (c) 2025 Shreos Roychowdhury.
# Copyright (c) 2025 Tirja Consulting LLP.
# 
# This source code is protected under international copyright law.  All rights
# reserved and protected by the copyright holders.
#
# This file is confidential and only available to authorized individuals with the
# permission of the copyright holders.  If you encounter this file and do not have
# permission, please contact the copyright holders and delete this file.
#
# THE SOFTWARE IS PROVIDED , WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
# FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
# COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
# IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
# 

import os
import json
from datetime import datetime
import hashlib
from enum import Enum, IntEnum
from tirjapy.utils.TypedEnum import TypedEnum

from tirjapy.utils.HandleQuotes import HandleQuotes

class TaskState(metaclass=TypedEnum):
	NONE                        = ''
	QUEUED                      = 'queued'
	ONGOING                     = 'ongoing'
	DONE                        = 'done'
	FAILED                      = 'failed'
	ACCEPTED                    = 'accepted'
	NOTFOUND                    = 'notfound'
	CANCELLED                   = 'cancelled'

class StoreBase(HandleQuotes):

	def __init__(self):
		""" init fx inits handles """
		pass

	def _SaveLocalFile(self, xfile, savepath):
		""" Writes to disk with recursive mkdir"""
		dirname = os.path.dirname(savepath)
		if not os.path.exists(dirname):
				os.makedirs(dirname,exist_ok = True)
		xfile.save( savepath )

	def _CleanUpFile(self, fname):
		""" recursively clean file """
		## delete files
		dname = fname
		try:
			os.remove(dname)
			dname = os.path.dirname(dname)
			while len(dname) > 1:
				os.rmdir(dname)
				dname = os.path.dirname(dname)
		except:
			pass
		print("Cleanup: Till : %s" % dname , flush=True)

	def GetData(self, blank=False):
		""" TO INHERIT: Prints Json of class """
		raise ValueError("GetData not implemented in StoreBase")

	def _GetHashOne(self, one):
		return hashlib.sha256(one.encode()).hexdigest()

	def _GetHashTwo(self, one, two):
		return hashlib.sha256('|'.join([one,two]).encode()).hexdigest()




# -*- coding: utf-8 -*-
#
# @project TirjaPy
# @file src/tirjapy/utils/StorageHandle.py
# @author  Shreos Roychowdhury <shreos@tirja.com>
# @version 1.0.0
# 
# @section DESCRIPTION
# 
#   StorageHandle.py : AWS S3 handlers
# 
# @section LICENSE
# 
# Copyright (c) 2025 Shreos Roychowdhury.
# Copyright (c) 2025 Tirja Consulting LLP.
# 
# This source code is protected under international copyright law.  All rights
# reserved and protected by the copyright holders.
#
# This file is confidential and only available to authorized individuals with the
# permission of the copyright holders.  If you encounter this file and do not have
# permission, please contact the copyright holders and delete this file.
#
# THE SOFTWARE IS PROVIDED , WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
# FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
# COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
# IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
# 

import os
import sys
import json
import boto3
from boto3.s3.transfer import S3Transfer
from mimetypes import MimeTypes

from tirjapy.utils.HandleQuotes import HandleQuotes

MIME_PPTX = 'application/vnd.openxmlformats-officedocument.presentationml.presentation'
MIME_JSON = 'application/json'
MIME_ZIP = 'application/zip'
MIME_PPT = 'application/vnd.ms-powerpoint'
MIME_PDF = 'application/pdf'
MIME_TXT = 'text/plain'
MIME_GZIP = 'application/x-gzip'

MIME_PNG = 'image/png'
MIME_UNKNOWN = 'application/octet-stream'

MIME_IMAGETYPES = {
	'png' : 'image/png',
	'svg' : 'image/svg+xml',
	'jpg' : 'image/jpeg',
	'jpeg' : 'image/jpeg',
	'gif' : 'image/gif',
	'bmp' : 'image/bmp'
}

MIME_OTHERTYPES = {
	'pptx' : MIME_PPTX,
	'json' : MIME_JSON,
	'zip'  : MIME_ZIP,
	'ppt'  : MIME_PPT,
	'pdf'  : MIME_PDF,
	'txt'  : MIME_TXT,
	'gz'  : MIME_GZIP
}

class StorageHandle(HandleQuotes):

	s3_creds = None

	def __init__(self):
		""" constructor default"""
		self.is_init = False
		if StorageHandle.s3_creds:
			self._Initialize()

	def _SanityCheck(self):
		""" check if globals set """
		if not self.is_init:
			raise ValueError("S3 Credentials not set in StorageHandle")

	def _ValidateDataObj(self, data):
		""" check if data is ok """
		if not data.s3_bucket:
			raise ValueError("s3_bucket not set")
		if not data.s3_path:
			raise ValueError("s3_path not set")
		if not data.local_path:
			raise ValueError("local_path not set")

	def RegisterGlobals(self, params):
		"""Register Global fx"""

		StorageHandle.s3_creds = {
			'access_key' : self._RequiredField( params , 'access_key' ),
			'secret_key' : self._RequiredField( params , 'secret_key' ),
			'bucket' : self._RequiredField( params , 'bucket' ),
			'region' : self._RequiredField( params , 'region' ),
			'bucket2' : self._OptionalField( params , 'bucket2' ),
			'region2' : self._OptionalField( params , 'region2' ),
			'format' : self._OptionalField( params , 'format' , 'json'),
		}
		self._Initialize()

	def _Initialize(self):
		""" Init fx"""
		self.bucket = StorageHandle.s3_creds['bucket']
		self.client = boto3.client('s3', StorageHandle.s3_creds['region'],
			aws_access_key_id=StorageHandle.s3_creds['access_key'],
			aws_secret_access_key=StorageHandle.s3_creds['secret_key']
		)
		self.transfer = S3Transfer( self.client )

		## set a second bucket if valid
		self.has_two_buckets = False
		if ( StorageHandle.s3_creds['bucket2'] and StorageHandle.s3_creds['region2'] ):
			self.has_two_buckets = True
			self.bucket2 = StorageHandle.s3_creds['bucket2']
			self.transfer2 = S3Transfer(
				boto3.client('s3', StorageHandle.s3_creds['region2'],
					aws_access_key_id=StorageHandle.s3_creds['access_key'],
					aws_secret_access_key=StorageHandle.s3_creds['secret_key']
				)
			)
			print("S3 Storage : configured two bucket", flush=True)
		## is init
		self.is_init = True

	def _GetMimeType(self, filename):
		""" Gets the mime type """
		extn = self._GetLast(filename, '.' )
		if extn in MIME_IMAGETYPES:
			return MIME_IMAGETYPES[extn]
		elif extn in MIME_OTHERTYPES:
			return MIME_OTHERTYPES[extn]
		return MIME_UNKNOWN

	def GetBucket(self):
		""" Gets the bucket """
		if 'bucket' not in self.__dict__:
			print("S3 bucket lookup : FAILED", flush=True)
			raise ValueError("Bucket not set yet")
		return self.bucket

	def WriteFile(self, loc, rem, mime_type):
		""" Writes file to S3 both bucket and if present bucket2"""
		self._SanityCheck()
		extra_args={'ContentType' : mime_type}
		self.transfer.upload_file(loc, self.bucket, rem, extra_args=extra_args)
		if self.has_two_buckets:
			self.transfer2.upload_file(loc, self.bucket2, rem, extra_args=extra_args)

	def ReadFile(self, bucket, s3path, savepath):
		""" Reads file from S3 from bucket only"""
		self._SanityCheck()
		dirname = os.path.dirname(savepath)
		if not os.path.exists(dirname):
				os.makedirs(dirname,exist_ok = True)
		self.transfer.download_file(bucket, s3path, savepath)

	def WriteFileObj(self, data):
		""" Writes file to S3 using FileStore Object as data"""
		self._SanityCheck()
		self._ValidateDataObj(data)
		## take mime type from extn
		mime_type = self._GetMimeType( data.s3_path )
		extra_args={'ContentType' : mime_type}
		## copy
		self.transfer.upload_file(data.local_path, data.s3_bucket, data.s3_path, extra_args=extra_args)

	def ReadFileObj(self, data):
		""" Reads file from S3 using FileStore Object as data"""
		self._SanityCheck()
		self._ValidateDataObj(data)
		## copy
		self.transfer.download_file(data.s3_bucket, data.s3_path, data.local_path)




# -*- coding: utf-8 -*-
#
# @project TirjaPy
# @file src/tirjapy/utils/JcsvData.py
# @author  Shreos Roychowdhury <shreos@tirja.com>
# @version 1.0.0
# 
# @section DESCRIPTION
# 
#   JcsvData.py : Handle jcsv data
# 
# @section LICENSE
# 
# Copyright (c) 2025 Shreos Roychowdhury.
# Copyright (c) 2025 Tirja Consulting LLP.
# 
# This source code is protected under international copyright law.  All rights
# reserved and protected by the copyright holders.
#
# This file is confidential and only available to authorized individuals with the
# permission of the copyright holders.  If you encounter this file and do not have
# permission, please contact the copyright holders and delete this file.
#
# THE SOFTWARE IS PROVIDED , WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
# FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
# COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
# IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
# 

import os
import sys
import json

from tirjapy.utils.HandleQuotes import HandleQuotes

class JcsvData(HandleQuotes):
	"""Class JcsvData handles jcsv data """

	def __init__(self, headers, data=None):
		""" Read from headers and data """
		if not isinstance(headers, list):
			raise ValueError("Jcsv: headers is not array")
		## store positions
		self.positions = {}
		pos = 0
		for item in headers:
			self.positions[item]=pos
			pos = pos + 1
		## init data array
		self.data = []
		if data:
			self.AddRows(data)

	def _AddRow(self, data):
		""" Add a single data unit """
		if len(data) != len(self.positions):
			raise ValueError("Jcsv _AddRow: data array has wrong length")
		self.data.append(data)

	def AddRows(self, data):
		""" Add several data unit """
		for item in data:
			self._AddRow(item)

	def GetData(self, row, cname):
		""" Get one data point by row and name"""
		if row >= len(self.data):
			raise ValueError("Jcsv GetData: exceeds no of rows")
		if cname not in self.positions:
			raise ValueError("Jcsv GetData: no such col")
		return self.data[row][ self.positions[cname] ]

	def GetCols(self, row, cnames):
		""" Get multiple data points by row and names"""
		outp = []
		for nm in cnames:
			outp.append ( self.GetData(row, nm) )
		return outp

	def _SetData(self, row, cname, cval):
		""" Set one data point by row and name"""
		if row >= len(self.data):
			raise ValueError("Jcsv GetData: exceeds no of rows")
		if cname not in self.positions:
			raise ValueError("Jcsv GetData: no such col")
		self.data[row][ self.positions[cname] ] = cval

	def SetData(self, row, data):
		""" Set one data point by row and data"""
		for item in data.keys():
			self._SetData(row, item, data[item] )

	def AddRowData(self, data):
		""" Add one data point by data object ignore unknown and missing"""
		darray = [''] * self._NoofCols()
		for cname in data.keys():
			if cname in self.positions:
				darray[ self.positions[cname] ] = str(data[cname])
		self._AddRow(darray)

	def _NoofRows(self):
		""" Get no of rows """
		return len(self.data)

	def _NoofCols(self):
		""" Get no of cols """
		return len(self.positions)

	def GetHeaders(self):
		""" get headers as array """
		headers = [''] * len(self.positions)
		for item in self.positions.keys():
			headers[ self.positions[item] ] = item
		return headers

	def GetHeadersStringB(self):
		""" get headers as back quoted string """
		headers = [''] * len(self.positions)
		for item in self.positions.keys():
			headers[ self.positions[item] ] = self._AddBackQ(item)
		return ','.join(headers)

	def GetRow(self, row):
		""" get one row data """
		if row >= len(self.data):
			raise ValueError("Jcsv GetData: exceeds no of rows")
		return self.data[row]

	def GetRows(self):
		""" get all row data """
		return self.data

	def GetRowStringQ(self, row):
		""" get one row data """
		if row >= len(self.data):
			raise ValueError("Jcsv GetData: exceeds no of rows")
		rows = []
		for item in self.data[row]:
			rows.append( self._AddQ(item) )
		return ','.join(rows)

	def GetAll(self):
		""" As data """
		return { 'headers' : self.GetHeaders(), 'data' : self.GetRows() }



# -*- coding: utf-8 -*-
#
# @project TirjaPy
# @file src/tirjapy/utils/RedisHandle.py
# @author  Shreos Roychowdhury <shreos@tirja.com>
# @version 1.0.0
# 
# @section DESCRIPTION
# 
#   RedisHandle.py : Redis handlers
# 
# @section LICENSE
# 
# Copyright (c) 2025 Shreos Roychowdhury.
# Copyright (c) 2025 Tirja Consulting LLP.
# 
# This source code is protected under international copyright law.  All rights
# reserved and protected by the copyright holders.
#
# This file is confidential and only available to authorized individuals with the
# permission of the copyright holders.  If you encounter this file and do not have
# permission, please contact the copyright holders and delete this file.
#
# THE SOFTWARE IS PROVIDED , WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
# FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
# COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
# IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
# 

import os
import sys
import json
from redis import Redis
from rq import Worker, Queue, Connection
from rq.job import Job, JobStatus

from tirjapy.utils.HandleQuotes import HandleQuotes
from tirjapy.base.StoreBase import TaskState
from tirjapy.base.HandleConstants import TASK_CANCEL_HSET, TASK_DO_CANCEL

class RedisHandle(HandleQuotes):

	rds_creds = None

	def __init__(self):
		""" constructor default"""
		self.is_init = False
		if RedisHandle.rds_creds:
			self._Initialize()

	def _SanityCheck(self):
		""" check if globals set """
		if not self.is_init:
			raise ValueError("Globals not set for Redis Handle")

	def RegisterGlobals(self, params):
		"""Register Global fx"""

		RedisHandle.rds_creds = {
			'host' : self._RequiredField(params, 'host'),
			'port' : self._RequiredField(params, 'port'),
			'queue' : self._RequiredField(params, 'queue'),
		}
		self._Initialize()

	def _Initialize(self):
		""" Init fx"""
		self.conn = Redis(host=RedisHandle.rds_creds['host'], port=RedisHandle.rds_creds['port'])
		self.queue = RedisHandle.rds_creds['queue']
		self.is_init = True

	def GetRedisUrl(self):
		""" Get redis url """
		self._SanityCheck()
		return 'redis://' + RedisHandle.rds_creds['host'] + ':' + RedisHandle.rds_creds['port']

	def GetConn(self):
		""" Get redis conn """
		self._SanityCheck()
		return self.conn

	def GetQueue(self):
		""" Get redis queue """
		self._SanityCheck()
		return self.queue

	def GetJobStatus(self, queue_id):
		""" gets redis task by queue_id, return state and job """
		self._SanityCheck()
		## check if key exists in redis
		if not Job.exists(queue_id, connection=self.conn):
			return TaskState.NOTFOUND
		## check job status
		job = Job.fetch(queue_id, connection=self.conn)
		job_status = job.get_status(refresh=True)
		task_state_map = {
			JobStatus.QUEUED     :  TaskState.QUEUED,
			JobStatus.FAILED     :  TaskState.FAILED,
			JobStatus.STARTED    :  TaskState.ONGOING,
			JobStatus.FINISHED   :  TaskState.DONE,
			JobStatus.DEFERRED   :  TaskState.ACCEPTED,
			JobStatus.SCHEDULED  :  TaskState.ACCEPTED
		}
		task_state = task_state_map[job_status] if job_status in task_state_map else TaskState.NOTFOUND
		return task_state

	def SetCancelByJobKey(self, queue_id):
		""" set cancel redis task by queue_id, return true/false or throw """
		self._SanityCheck()
		job = Job.fetch(queue_id, connection=self.conn)
		job_status = job.get_status(refresh=True)
		## cancel it queued, set if started
		if job_status == JobStatus.QUEUED or job_status == JobStatus.DEFERRED or job_status == JobStatus.SCHEDULED:
			job.delete()
			return True
		elif job_status == JobStatus.STARTED:
			self.conn.hset( TASK_CANCEL_HSET, queue_id, TASK_DO_CANCEL )
			return False
		## raise exception if reaches here
		raise ValueError("Cannot Cancel this job")

	def CheckCancelByJobKey(self, queue_id, dont_delete=False):
		""" check if task is cancelled """
		self._SanityCheck()
		if self.conn.hget( TASK_CANCEL_HSET, queue_id) == TASK_DO_CANCEL:
			if not dont_delete:
				self.conn.hdel( TASK_CANCEL_HSET, queue_id)
			return True
		return False

	def _GetQData(self):
		""" check all queue data """
		self._SanityCheck()
		return {
			q.name: {
				'name': q.name,
				'queued': len(q.job_ids)
			} for q in Queue.all(connection=self.conn)}

	def _GetQWorkers(self):
		""" check all worker data """
		self._SanityCheck()
		return [{
			'name': w.name,
			'hostname': w.hostname,
			'pid': w.pid,
			'queues': ", ".join([q.name for q in w.queues]),
			'state': w.state,
			'last_heartbeat': w.last_heartbeat,
			'start_date': w.birth_date,
			'successful_jobs': w.successful_job_count,
			'failed_jobs': w.failed_job_count,
			'total_working_time': w.total_working_time
		} for w in Worker.all(connection=self.conn)]


	def GetInfo(self):
		""" get web info """
		return {
			'queues': self._GetQData(),
			'workers': self._GetQWorkers()
		}




# -*- coding: utf-8 -*-
#
# @project TirjaPy
# @file src/tirjapy/utils/HtPasswdAuth.py
# @author  Shreos Roychowdhury <shreos@tirja.com>
# @version 1.0.0
# 
# @section DESCRIPTION
# 
#   HtPasswdAuth.py : Decorator, configuration, and error handler for
# 
# @section LICENSE
# 
# Copyright (c) 2025 Shreos Roychowdhury.
# Copyright (c) 2025 Tirja Consulting LLP.
# 
# This source code is protected under international copyright law.  All rights
# reserved and protected by the copyright holders.
#
# This file is confidential and only available to authorized individuals with the
# permission of the copyright holders.  If you encounter this file and do not have
# permission, please contact the copyright holders and delete this file.
#
# THE SOFTWARE IS PROVIDED , WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
# FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
# COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
# IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
# 

from __future__ import absolute_import, unicode_literals
from functools import wraps
import hashlib
import jwt
import logging

from flask import request, Response, current_app, g
from passlib.apache import HtpasswdFile


log = logging.getLogger(__name__)  # pylint: disable=invalid-name


class HtPasswdAuth:
	"""Configure htpasswd based basic and token authentication."""

	def __init__(self, app=None):
		"""Boiler plate extension init with log_level being declared"""
		self.users = HtpasswdFile()
		self.app = app
		if app is not None:
			self.init_app(app)

	def init_app(self, app):
		"""
		Find and configure the user database from specified file
		"""
		# pylint: disable=inconsistent-return-statements
		app.config.setdefault('FLASK_AUTH_ALL', False)
		app.config.setdefault('FLASK_AUTH_REALM', 'Login Required')
		# Default set to bad file to trigger IOError
		app.config.setdefault('FLASK_HTPASSWD_PATH', '/^^^/^^^')

		# Load up user database
		try:
			self.load_users(app)
		except IOError:
			log.critical(
				'No htpasswd file loaded, please set `FLASK_HTPASSWD`'
				'or `FLASK_HTPASSWD_PATH` environment variable to a '
				'valid apache htpasswd file.'
			)

		# Allow requiring auth for entire app, with pre request method
		@app.before_request
		def require_auth():
			# pylint: disable=unused-variable
			"""Pre request processing for enabling full app authentication."""
			if not current_app.config['FLASK_AUTH_ALL']:
				return None
			is_valid, user = self.authenticate()
			if not is_valid:
				return self.auth_failed()
			g.user = user

	def load_users(self, app):
		"""
		Load users from configured file.

		Args:
			app (flask.Flask): Flask application to load users from.

		Raises:
			IOError: If the configured htpasswd file does not exist.
		Returns:
			None
		"""
		self.users = HtpasswdFile(
			app.config['FLASK_HTPASSWD_PATH']
		)

	def check_basic_auth(self, username, password):
		"""
		This function is called to check if a username /
		password combination is valid via the htpasswd file.
		"""
		valid = self.users.check_password(
			username, password
		)
		if not valid:
			log.warning('Invalid login from %s', username)
			valid = False
		return (
			valid,
			username
		)

	def get_signature(self):
		"""
		Setup crypto sig.
		"""
		with self.app.app_context():
			return current_app.config['FLASK_SECRET']

	def get_hashhash(self, username):
		"""
		Generate a digest of the htpasswd hash
		"""
		userhash = self.users.get_hash(username)
		if not userhash:
			return ''
		return hashlib.sha256(userhash).hexdigest()

	def generate_token(self, username):
		"""
		assumes user exists in htpasswd file.

		Return the token for the given user by signing a token of
		the username and a hash of the htpasswd string.
		"""
		key = self.get_signature()
		return jwt.encode({
			'username': username,
			'hashhash': self.get_hashhash(username)
		}, key, algorithm="HS512")

	def check_token_auth(self, token):
		"""
		Check to see who this is and if their token gets
		them into the system.
		"""
		key = self.get_signature()
		try:
			data = jwt.decode(token, key, algorithms=["HS512"])
		except:
			log.warning('Received bad token signature')
			return False, None
		if data['username'] not in self.users.users():
			log.warning(
				'Token auth signed message, but invalid user %s',
				data['username']
			)
			return False, None
		if data['hashhash'] != self.get_hashhash(data['username']):
			log.warning(
				'Token and password do not match, %s '
				'needs to regenerate token',
				data['username']
			)
			return False, None
		return True, data['username']

	@staticmethod
	def auth_failed():
		"""
		Sends a 401 response that enables basic auth
		"""
		return Response(
			'Could not verify your access level for that URL.\n'
			'You have to login with proper credentials',
			401,
			{'WWW-Authenticate': 'Basic realm="{0}"'.format(
				current_app.config['FLASK_AUTH_REALM']
			)}
		)

	def authenticate(self):
		"""Authenticate user by any means and return either true or false.

		Args:

		Returns:
			tuple (is_valid, username): True is valid user, False if not
		"""
		basic_auth = request.authorization
		is_valid = False
		user = None
		if basic_auth:
			is_valid, user = self.check_basic_auth(
				basic_auth.username, basic_auth.password
			)
		else:  # Try token auth
			token = request.headers.get('Authorization', None)
			param_token = request.args.get('access_token')
			if token or param_token:
				if token:
					# slice the 'token ' piece of the header (following
					# github style):
					token = token[6:]
				else:
					# Grab it from query dict instead
					token = param_token
				log.debug('Received token: %s', token)

				is_valid, user = self.check_token_auth(token)
		return (is_valid, user)

	def required(self, func):
		"""
		Decorator function with basic and token authentication handler
		"""
		@wraps(func)
		def decorated(*args, **kwargs):
			"""
			Actual wrapper to run the auth checks.
			"""
			is_valid, user = self.authenticate()
			if not is_valid:
				return self.auth_failed()
			kwargs['user'] = user
			return func(*args, **kwargs)
		return decorated

	def async_required(self, func):
		"""
		Decorator function with basic and token authentication handler for async
		"""
		@wraps(func)
		async def decorated(*args, **kwargs):
			""" Actual wrapper to run the auth checks.  """
			is_valid, user = self.authenticate()
			if not is_valid:
				return self.auth_failed()
			kwargs['user'] = user
			return await func(*args, **kwargs)
		return decorated
